@@ -15,7 +15,7 @@
  */
 package org.gradle.buildinit.plugins.internal;
 
-import org.gradle.api.UncheckedIOException;
+import org.gradle.internal.UncheckedException;
 
 import java.io.IOException;
 import java.util.Properties;
@@ -28,7 +28,7 @@ public class DefaultTemplateLibraryVersionProvider implements TemplateLibraryVer
         try {
             this.libraryVersions.load(getClass().getResourceAsStream("/org/gradle/buildinit/tasks/templates/library-versions.properties"));
         } catch (IOException e) {
-            throw new UncheckedIOException(e);
+            throw UncheckedException.throwAsUncheckedException(e);
         }
     }
 
