@@ -23,11 +23,17 @@ import org.gradle.api.internal.file.FileCollectionFactory;
 import org.gradle.api.internal.file.FilePropertyFactory;
 import org.gradle.api.internal.file.collections.DirectoryFileTreeFactory;
 import org.gradle.api.internal.file.collections.FileCollectionObservationListener;
+import org.gradle.api.internal.initialization.BuildLogicBuildQueue;
 import org.gradle.api.internal.initialization.DefaultBuildLogicBuildQueue;
 import org.gradle.api.internal.model.DefaultObjectFactory;
 import org.gradle.api.internal.model.NamedObjectInstantiator;
 import org.gradle.api.internal.project.DefaultProjectStateRegistry;
+import org.gradle.api.internal.project.ProjectStateRegistry;
 import org.gradle.api.internal.project.taskfactory.TaskIdentityFactory;
+import org.gradle.api.internal.properties.DefaultGradlePropertiesController;
+import org.gradle.api.internal.properties.GradlePropertiesController;
+import org.gradle.api.internal.properties.GradlePropertiesListener;
+import org.gradle.api.internal.provider.ConfigurationTimeBarrier;
 import org.gradle.api.internal.provider.DefaultConfigurationTimeBarrier;
 import org.gradle.api.internal.provider.PropertyFactory;
 import org.gradle.api.internal.tasks.TaskDependencyFactory;
@@ -35,24 +41,30 @@ import org.gradle.api.logging.configuration.LoggingConfiguration;
 import org.gradle.api.logging.configuration.ShowStacktrace;
 import org.gradle.api.model.ObjectFactory;
 import org.gradle.api.problems.internal.InternalProblems;
-import org.gradle.api.tasks.util.PatternSet;
+import org.gradle.api.tasks.util.internal.PatternSetFactory;
 import org.gradle.configuration.project.BuiltInCommand;
 import org.gradle.execution.DefaultTaskSelector;
 import org.gradle.execution.ProjectConfigurer;
 import org.gradle.execution.TaskNameResolver;
 import org.gradle.execution.TaskPathProjectEvaluator;
 import org.gradle.execution.TaskSelector;
+import org.gradle.execution.selection.BuildTaskSelector;
 import org.gradle.execution.selection.DefaultBuildTaskSelector;
 import org.gradle.initialization.BuildOptionBuildOperationProgressEventsEmitter;
+import org.gradle.initialization.Environment;
 import org.gradle.initialization.exception.DefaultExceptionAnalyser;
 import org.gradle.initialization.exception.ExceptionCollector;
 import org.gradle.initialization.exception.MultipleBuildFailuresExceptionAnalyser;
 import org.gradle.initialization.exception.StackTraceSanitizingExceptionAnalyser;
-import org.gradle.internal.Factory;
+import org.gradle.initialization.properties.DefaultGradlePropertiesLoader;
+import org.gradle.initialization.properties.DefaultSystemPropertiesInstaller;
+import org.gradle.initialization.properties.SystemPropertiesInstaller;
+import org.gradle.internal.build.BuildLifecycleControllerFactory;
 import org.gradle.internal.build.BuildStateRegistry;
 import org.gradle.internal.build.DefaultBuildLifecycleControllerFactory;
 import org.gradle.internal.buildoption.DefaultFeatureFlags;
 import org.gradle.internal.buildoption.DefaultInternalOptions;
+import org.gradle.internal.buildoption.FeatureFlags;
 import org.gradle.internal.buildoption.InternalOptions;
 import org.gradle.internal.enterprise.core.GradleEnterprisePluginManager;
 import org.gradle.internal.event.ListenerManager;
@@ -64,7 +76,6 @@ import org.gradle.internal.instrumentation.reporting.DefaultMethodInterceptionRe
 import org.gradle.internal.instrumentation.reporting.ErrorReportingMethodInterceptionReportCollector;
 import org.gradle.internal.instrumentation.reporting.MethodInterceptionReportCollector;
 import org.gradle.internal.instrumentation.reporting.PropertyUpgradeReportConfig;
-import org.gradle.internal.model.BuildTreeObjectFactory;
 import org.gradle.internal.problems.DefaultProblemDiagnosticsFactory;
 import org.gradle.internal.scopeids.id.BuildInvocationScopeId;
 import org.gradle.internal.service.PrivateService;
@@ -73,6 +84,8 @@ import org.gradle.internal.service.ServiceRegistration;
 import org.gradle.internal.service.ServiceRegistrationProvider;
 import org.gradle.internal.service.scopes.GradleModuleServices;
 import org.gradle.internal.service.scopes.Scope;
+import org.gradle.problems.buildtree.ProblemDiagnosticsFactory;
+import org.gradle.problems.buildtree.ProblemReporter;
 
 import java.util.List;
 
@@ -97,25 +110,25 @@ public class BuildTreeScopeServices implements ServiceRegistrationProvider {
         registration.add(BuildInvocationScopeId.class, buildInvocationScopeId);
         registration.add(BuildTreeState.class, buildTree);
         registration.add(GradleEnterprisePluginManager.class);
-        registration.add(DefaultBuildLifecycleControllerFactory.class);
+        registration.add(BuildLifecycleControllerFactory.class, DefaultBuildLifecycleControllerFactory.class);
         registration.add(BuildOptionBuildOperationProgressEventsEmitter.class);
         registration.add(BuildInclusionCoordinator.class);
-        registration.add(DefaultProjectStateRegistry.class);
-        registration.add(DefaultConfigurationTimeBarrier.class);
-        registration.add(DeprecationsReporter.class);
-        registration.add(TaskPathProjectEvaluator.class);
-        registration.add(DefaultFeatureFlags.class);
-        registration.add(DefaultProblemDiagnosticsFactory.class);
-        registration.add(DefaultExceptionAnalyser.class);
+        registration.add(ProjectStateRegistry.class, DefaultProjectStateRegistry.class);
+        registration.add(ConfigurationTimeBarrier.class, DefaultConfigurationTimeBarrier.class);
+        registration.add(ProblemReporter.class, DeprecationsReporter.class);
+        registration.add(ProjectConfigurer.class, TaskPathProjectEvaluator.class);
+        registration.add(FeatureFlags.class, DefaultFeatureFlags.class);
+        registration.add(ProblemDiagnosticsFactory.class, DefaultProblemDiagnosticsFactory.class);
+        registration.add(ExceptionCollector.class, DefaultExceptionAnalyser.class);
         registration.add(ConfigurationCacheableIdFactory.class);
         registration.add(TaskIdentityFactory.class);
-        registration.add(DefaultBuildLogicBuildQueue.class);
+        registration.add(BuildLogicBuildQueue.class, DefaultBuildLogicBuildQueue.class);
         modelServices.applyServicesTo(registration);
     }
 
     @Provides
-    BuildTreeObjectFactory createObjectFactory(
-        InstantiatorFactory instantiatorFactory, DirectoryFileTreeFactory directoryFileTreeFactory, Factory<PatternSet> patternSetFactory,
+    ObjectFactory createObjectFactory(
+        InstantiatorFactory instantiatorFactory, DirectoryFileTreeFactory directoryFileTreeFactory, PatternSetFactory patternSetFactory,
         PropertyFactory propertyFactory, FilePropertyFactory filePropertyFactory, TaskDependencyFactory taskDependencyFactory, FileCollectionFactory fileCollectionFactory,
         DomainObjectCollectionFactory domainObjectCollectionFactory, NamedObjectInstantiator instantiator
     ) {
@@ -131,20 +144,18 @@ public class BuildTreeScopeServices implements ServiceRegistrationProvider {
             domainObjectCollectionFactory);
     }
 
-
-
     @Provides
     protected InternalOptions createInternalOptions(StartParameter startParameter) {
         return new DefaultInternalOptions(startParameter.getSystemPropertiesArgs());
     }
 
     @Provides
-    protected TaskSelector createTaskSelector(ProjectConfigurer projectConfigurer, ObjectFactory objectFactory) {
-        return objectFactory.newInstance(DefaultTaskSelector.class, new TaskNameResolver(), projectConfigurer);
+    protected TaskSelector createTaskSelector(ObjectFactory objectFactory) {
+        return objectFactory.newInstance(DefaultTaskSelector.class, new TaskNameResolver());
     }
 
     @Provides
-    protected DefaultBuildTaskSelector createBuildTaskSelector(BuildStateRegistry buildRegistry, TaskSelector taskSelector, List<BuiltInCommand> commands, InternalProblems problemsService) {
+    protected BuildTaskSelector createBuildTaskSelector(BuildStateRegistry buildRegistry, TaskSelector taskSelector, List<BuiltInCommand> commands, InternalProblems problemsService) {
         return new DefaultBuildTaskSelector(buildRegistry, taskSelector, commands, problemsService);
     }
 
@@ -182,4 +193,23 @@ public class BuildTreeScopeServices implements ServiceRegistrationProvider {
             startParameter.isPropertyUpgradeReportEnabled()
         );
     }
+
+    @Provides
+    protected SystemPropertiesInstaller createSystemPropertiesInstaller(StartParameterInternal startParameter) {
+        return new DefaultSystemPropertiesInstaller(startParameter);
+    }
+
+    @Provides
+    protected GradlePropertiesController createGradlePropertiesController(
+        StartParameterInternal startParameter,
+        Environment environment,
+        SystemPropertiesInstaller systemPropertiesInstaller,
+        ListenerManager listenerManager
+    ) {
+        return new DefaultGradlePropertiesController(
+            new DefaultGradlePropertiesLoader(startParameter, environment),
+            systemPropertiesInstaller,
+            listenerManager.getBroadcaster(GradlePropertiesListener.class)
+        );
+    }
 }
