@@ -16,9 +16,12 @@
 package org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy;
 
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.Versioned;
+import org.gradle.internal.service.scopes.Scope;
+import org.gradle.internal.service.scopes.ServiceScope;
 
 import java.util.Comparator;
 
+@ServiceScope(Scope.Build.class)
 public interface VersionComparator extends Comparator<Versioned> {
     /**
      * Compares two versioned elements to see which is the 'latest'.
