@@ -13,5 +13,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-@org.gradle.api.NonNullApi
+@NullMarked
 package org.gradle.api.internal.artifacts.transform;
+
+import org.jspecify.annotations.NullMarked;
