@@ -17,18 +17,30 @@
 package org.gradle.kotlin.dsl.provider.plugins.precompiled.tasks
 
 import org.gradle.StartParameter
-import org.gradle.api.GradleException
-import org.gradle.api.file.ConfigurableFileCollection
+import org.gradle.api.artifacts.ArtifactCollection
+import org.gradle.api.artifacts.Configuration
+import org.gradle.api.artifacts.ConfigurationContainer
+import org.gradle.api.artifacts.component.ProjectComponentIdentifier
+import org.gradle.api.artifacts.dsl.DependencyHandler
 import org.gradle.api.file.DirectoryProperty
+import org.gradle.api.file.FileCollection
 import org.gradle.api.file.ProjectLayout
 import org.gradle.api.internal.StartParameterInternal
+import org.gradle.api.internal.artifacts.DependencyManagementServices
+import org.gradle.api.internal.artifacts.dependencies.DefaultFileCollectionDependency
+import org.gradle.api.internal.artifacts.dsl.dependencies.DependencyFactoryInternal.ClassPathNotation
+import org.gradle.api.internal.file.FileCollectionFactory
+import org.gradle.api.internal.initialization.ScriptClassPathResolver
+import org.gradle.api.internal.initialization.StandaloneDomainObjectContext
 import org.gradle.api.internal.project.ProjectInternal
 import org.gradle.api.internal.project.ProjectStateRegistry
+import org.gradle.api.invocation.Gradle
+import org.gradle.api.provider.ListProperty
 import org.gradle.api.provider.Property
+import org.gradle.api.provider.Provider
 import org.gradle.api.tasks.CacheableTask
 import org.gradle.api.tasks.Classpath
 import org.gradle.api.tasks.IgnoreEmptyDirectories
-import org.gradle.api.tasks.Input
 import org.gradle.api.tasks.InputDirectory
 import org.gradle.api.tasks.InputFiles
 import org.gradle.api.tasks.Internal
@@ -42,9 +54,9 @@ import org.gradle.initialization.ClassLoaderScopeRegistry
 import org.gradle.initialization.DefaultProjectDescriptor
 import org.gradle.internal.Try
 import org.gradle.internal.build.NestedRootBuildRunner.createNestedBuildTree
-import org.gradle.internal.classpath.CachedClasspathTransformer
 import org.gradle.internal.classpath.ClassPath
 import org.gradle.internal.classpath.DefaultClassPath
+import org.gradle.internal.component.local.model.OpaqueComponentIdentifier
 import org.gradle.internal.concurrent.CompositeStoppable.stoppable
 import org.gradle.internal.exceptions.LocationAwareException
 import org.gradle.internal.hash.HashCode
@@ -58,6 +70,7 @@ import org.gradle.kotlin.dsl.concurrent.AsyncIOScopeFactory
 import org.gradle.kotlin.dsl.concurrent.IO
 import org.gradle.kotlin.dsl.concurrent.writeFile
 import org.gradle.kotlin.dsl.precompile.PrecompiledScriptDependenciesResolver
+import org.gradle.kotlin.dsl.provider.plugins.precompiled.PrecompiledScriptException
 import org.gradle.kotlin.dsl.provider.plugins.precompiled.PrecompiledScriptPlugin
 import org.gradle.kotlin.dsl.provider.plugins.precompiled.scriptPluginFilesOf
 import org.gradle.kotlin.dsl.support.KotlinScriptType
@@ -69,7 +82,10 @@ import org.gradle.plugin.management.internal.PluginRequests
 import org.gradle.plugin.use.PluginDependenciesSpec
 import org.gradle.plugin.use.internal.PluginRequestApplicator
 import org.gradle.plugin.use.internal.PluginRequestCollector
+import org.gradle.util.internal.TextUtil.normaliseFileSeparators
+import java.io.ByteArrayOutputStream
 import java.io.File
+import java.io.PrintStream
 import java.net.URLClassLoader
 import java.nio.file.Files
 import javax.inject.Inject
@@ -97,7 +113,15 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
 
     @get:InputFiles
     @get:Classpath
-    abstract val runtimeClassPathFiles: ConfigurableFileCollection
+    val accessorsGenerationClassPathFiles: FileCollection
+        get() = accessorsGenerationClassPathArtifactCollection.get().artifactFiles
+
+    /**
+     * Tracked via [accessorsGenerationClassPathFiles].
+     */
+    @get:Internal
+    internal
+    abstract val accessorsGenerationClassPathArtifactCollection: Property<ArtifactCollection>
 
     @get:OutputDirectory
     abstract val metadataOutputDir: DirectoryProperty
@@ -109,27 +133,16 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
 
     @get:Internal
     internal
-    lateinit var plugins: List<PrecompiledScriptPlugin>
+    abstract val plugins: ListProperty<PrecompiledScriptPlugin>
 
     @get:InputFiles
     @get:IgnoreEmptyDirectories
     @get:PathSensitive(PathSensitivity.RELATIVE)
     @Suppress("unused")
     internal
-    val scriptFiles: Set<File>
+    val scriptFiles: Provider<Set<File>>
         get() = scriptPluginFilesOf(plugins)
 
-    @get:Input
-    abstract val strict: Property<Boolean>
-
-    init {
-        outputs.doNotCacheIf(
-            "Generated accessors can only be cached in strict mode."
-        ) {
-            !strict.get()
-        }
-    }
-
     /**
      *  ## Computation and sharing of type-safe accessors
      * 1. Group precompiled script plugins by the list of plugins applied in their `plugins` block.
@@ -146,10 +159,10 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
 
         recreateTaskDirectories()
 
-        val projectPlugins = selectProjectPlugins()
-        if (projectPlugins.isNotEmpty()) {
+        val projectScriptPlugins = selectProjectScriptPlugins()
+        if (projectScriptPlugins.isNotEmpty()) {
             asyncIOScopeFactory.newScope().useToRun {
-                generateTypeSafeAccessorsFor(projectPlugins)
+                generateTypeSafeAccessorsFor(projectScriptPlugins)
             }
         }
     }
@@ -162,8 +175,8 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
     }
 
     private
-    fun IO.generateTypeSafeAccessorsFor(projectPlugins: List<PrecompiledScriptPlugin>) {
-        resolvePluginGraphOf(projectPlugins)
+    fun IO.generateTypeSafeAccessorsFor(projectScriptPlugins: List<PrecompiledScriptPlugin>) {
+        resolvePluginGraphOf(projectScriptPlugins)
             .groupBy(
                 { it.appliedPlugins },
                 { it.scriptPlugin }
@@ -181,13 +194,13 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
     }
 
     private
-    fun resolvePluginGraphOf(projectPlugins: List<PrecompiledScriptPlugin>): Sequence<ScriptPluginPlugins> {
+    fun resolvePluginGraphOf(projectScriptPlugins: List<PrecompiledScriptPlugin>): Sequence<ScriptPluginPlugins> {
 
-        val scriptPluginsById = scriptPluginPluginsFor(projectPlugins).associateBy {
+        val scriptPluginsById = scriptPluginPluginsFor(projectScriptPlugins).associateBy {
             it.scriptPlugin.id
         }
 
-        val pluginGraph = plugins.associate {
+        val pluginGraph = plugins.get().associate {
             it.id to pluginsAppliedBy(it, scriptPluginsById)
         }
 
@@ -201,13 +214,11 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
         scriptPluginsById[scriptPlugin.id]?.appliedPlugins ?: emptyList()
 
     private
-    fun scriptPluginPluginsFor(projectPlugins: List<PrecompiledScriptPlugin>) = sequence {
+    fun scriptPluginPluginsFor(projectScriptPlugins: List<PrecompiledScriptPlugin>) = sequence {
         val loader = createPluginsClassLoader()
         try {
-            for (plugin in projectPlugins) {
-                loader.scriptPluginPluginsFor(plugin)?.let {
-                    yield(it)
-                }
+            for (plugin in projectScriptPlugins) {
+                yield(loader.scriptPluginPluginsFor(plugin))
             }
         } finally {
             stoppable(loader).stop()
@@ -215,21 +226,32 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
     }
 
     private
-    fun ClassLoader.scriptPluginPluginsFor(plugin: PrecompiledScriptPlugin): ScriptPluginPlugins? {
-
-        // The compiled script class won't be present for precompiled script plugins
-        // which don't include a `plugins` block
-        if (getResource(compiledScriptClassFile(plugin)) == null) {
-            return null
-        }
-
-        val pluginRequests = collectPluginRequestsOf(plugin)
-        validatePluginRequestsOf(plugin, pluginRequests)
-        return ScriptPluginPlugins(
-            plugin,
-            pluginRequests.map { it.id.id }
+    fun ClassLoader.scriptPluginPluginsFor(plugin: PrecompiledScriptPlugin): ScriptPluginPlugins =
+        withCapturedOutputOnError(
+            {
+                if (getResource(compiledScriptClassFile(plugin)) == null) {
+                    // The compiled script class won't be present for precompiled script plugins
+                    // which don't include a `plugins` block
+                    ScriptPluginPlugins(plugin, emptyList())
+                } else {
+                    val pluginRequests = collectPluginRequestsOf(plugin)
+                    validatePluginRequestsOf(plugin, pluginRequests)
+                    ScriptPluginPlugins(
+                        plugin,
+                        pluginRequests.map { it.id.id }
+                    )
+                }
+            },
+            { (error, stdout, stderr) ->
+                throw PrecompiledScriptException(
+                    buildString {
+                        append("Failed to collect plugin requests of '${projectRelativePathOf(plugin)}'")
+                        appendStdoutStderr(stdout, stderr)
+                    },
+                    error
+                )
+            }
         )
-    }
 
     private
     fun validatePluginRequestsOf(plugin: PrecompiledScriptPlugin, requests: PluginRequests) {
@@ -246,7 +268,8 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
     private
     fun validationErrorFor(pluginRequest: PluginRequestInternal): String? {
         if (pluginRequest.version != null) {
-            return "Invalid plugin request $pluginRequest. Plugin requests from precompiled scripts must not include a version number. Please remove the version from the offending request and make sure the module containing the requested plugin '${pluginRequest.id}' is an implementation dependency of $projectDesc."
+            return "Invalid plugin request $pluginRequest. Plugin requests from precompiled scripts must not include a version number. " +
+                "Please remove the version from the offending request and make sure the module containing the requested plugin '${pluginRequest.id}' is an implementation dependency of $projectDesc."
         }
         // TODO:kotlin-dsl validate apply false
         return null
@@ -281,7 +304,7 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
         plugin.compiledScriptTypeName.replace('.', '/') + ".class"
 
     private
-    fun selectProjectPlugins() = plugins.filter { it.scriptType == KotlinScriptType.PROJECT }
+    fun selectProjectScriptPlugins() = plugins.get().filter { it.scriptType == KotlinScriptType.PROJECT }
 
     private
     fun createPluginsClassLoader(): ClassLoader =
@@ -300,14 +323,17 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
     ): Map<HashedProjectSchema, List<PrecompiledScriptPlugin>> =
 
         pluginGroupsPerRequests.flatMap { (uniquePluginRequests, scriptPlugins) ->
-            try {
-                val schema = projectSchemaFor(pluginRequestsOf(scriptPlugins.first(), uniquePluginRequests)).get()
-                val hashed = HashedProjectSchema(schema)
-                scriptPlugins.map { hashed to it }
-            } catch (error: Throwable) {
-                reportProjectSchemaError(scriptPlugins, error)
-                emptyList()
-            }
+            withCapturedOutputOnError(
+                {
+                    val schema = projectSchemaFor(pluginRequestsOf(scriptPlugins.first(), uniquePluginRequests)).get()
+                    val hashed = HashedProjectSchema(schema)
+                    scriptPlugins.map { hashed to it }
+                },
+                { (error, stdout, stderr) ->
+                    reportProjectSchemaError(scriptPlugins, stdout, stderr, error)
+                    emptyList()
+                }
+            )
         }.groupBy(
             { (schema, _) -> schema },
             { (_, plugin) -> plugin }
@@ -319,29 +345,29 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
      */
     private
     fun projectSchemaFor(plugins: PluginRequests): Try<TypedProjectSchema> {
-        val buildLogicClassPath = buildLogicClassPath()
         val projectDir = uniqueTempDirectory()
         val startParameter = projectSchemaBuildStartParameterFor(projectDir)
         return createNestedBuildTree("$path:${projectDir.name}", startParameter, services).run { controller ->
             controller.withEmptyBuild { settings ->
                 Try.ofFailable {
                     val gradle = settings.gradle
-                    val baseScope = classLoaderScopeRegistry.coreAndPluginsScope.createChild("accessors-classpath").apply {
+                    val baseScope = classLoaderScopeRegistry.coreAndPluginsScope.createChild("accessors-classpath", null).apply {
                         // we export the build logic classpath to the base scope here so that all referenced plugins
                         // can be resolved in the root project scope created below.
-                        export(buildLogicClassPath)
+                        export(buildLogicClassPath(gradle))
                         lock()
                     }
-                    val rootProjectScope = baseScope.createChild("accessors-root-project")
+                    val rootProjectScope = baseScope.createChild("accessors-root-project", null)
                     settings.rootProject.name = "gradle-kotlin-dsl-accessors"
                     val projectState = gradle.serviceOf<ProjectStateRegistry>().registerProject(gradle.owner, settings.rootProject as DefaultProjectDescriptor)
                     projectState.createMutableModel(rootProjectScope, baseScope)
                     val rootProject = projectState.mutableModel
                     gradle.rootProject = rootProject
                     gradle.defaultProject = rootProject
+                    rootProject.projectEvaluationBroadcaster.beforeEvaluate(rootProject)
                     rootProject.run {
                         applyPlugins(plugins)
-                        serviceOf<ProjectSchemaProvider>().schemaFor(this)
+                        serviceOf<ProjectSchemaProvider>().schemaFor(this, classLoaderScope)!!
                     }
                 }
             }
@@ -357,7 +383,8 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
                     startParameter.gradleUserHomeDir,
                     projectDir,
                     projectDir
-                )
+                ),
+                startParameter.isOffline,
             )
         }
 
@@ -365,11 +392,15 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
      * A [StartParameter] subclass that provides no init scripts.
      */
     private
-    class ProjectSchemaBuildStartParameter(buildLayout: BuildLayoutParameters) : StartParameterInternal(buildLayout) {
+    class ProjectSchemaBuildStartParameter(
+        buildLayout: BuildLayoutParameters,
+        offline: Boolean,
+    ) : StartParameterInternal(buildLayout) {
 
         init {
             // Dry run in case a callback tries to access the task graph.
             isDryRun = true
+            isOffline = offline
             doNotSearchUpwards()
             useEmptySettings()
         }
@@ -380,12 +411,51 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
     }
 
     private
-    fun buildLogicClassPath(): ClassPath =
-        services.get<CachedClasspathTransformer>().transform(
-            DefaultClassPath.of(runtimeClassPathFiles),
-            CachedClasspathTransformer.StandardTransform.BuildLogic
+    fun buildLogicClassPath(gradle: Gradle): ClassPath {
+        // Ideally we would pass already instrumented classpath to a task and then just export it to the classloader.
+        // But since we do some artifact transform caching via BuildService,
+        // that would add some complexity when wiring GeneratePrecompiledScriptPluginAccessors task.
+        val dependencyManagementServices = gradle.serviceOf<DependencyManagementServices>()
+        val dependencyResolutionServices = dependencyManagementServices.newDetachedResolver(
+            StandaloneDomainObjectContext.PLUGINS
         )
 
+        val dependencies = dependencyResolutionServices.dependencyHandler
+        val configurations = dependencyResolutionServices.configurationContainer
+        val fileCollectionFactory = gradle.serviceOf<FileCollectionFactory>()
+        val configuration = createBuildLogicClassPathConfiguration(dependencies, configurations, fileCollectionFactory)
+
+        val resolver = gradle.serviceOf<ScriptClassPathResolver>()
+        val resolutionContext = resolver.prepareDependencyHandler(dependencies)
+        resolver.prepareClassPath(configuration, resolutionContext)
+        return resolver.resolveClassPath(configuration, resolutionContext)
+    }
+
+    private
+    fun createBuildLogicClassPathConfiguration(
+        dependencyHandler: DependencyHandler,
+        configurations: ConfigurationContainer,
+        fileCollectionFactory: FileCollectionFactory
+    ): Configuration {
+        val dependencies = accessorsGenerationClassPathArtifactCollection.get().artifacts.map {
+            when (val componentIdentifier = it.id.componentIdentifier) {
+                is OpaqueComponentIdentifier -> DefaultFileCollectionDependency(
+                    componentIdentifier,
+                    fileCollectionFactory.fixed(it.file)
+                )
+                is ProjectComponentIdentifier -> DefaultFileCollectionDependency(
+                    OpaqueComponentIdentifier(ClassPathNotation.LOCAL_PROJECT_AS_OPAQUE_DEPENDENCY),
+                    fileCollectionFactory.fixed(componentIdentifier.displayName, it.file)
+                )
+                else -> {
+                    dependencyHandler.create(fileCollectionFactory.fixed(it.file))
+                }
+            }
+        }.toTypedArray()
+        @Suppress("SpreadOperator")
+        return configurations.detachedConfiguration(*dependencies)
+    }
+
     private
     fun uniqueTempDirectory() =
         Files.createTempDirectory(temporaryDir.toPath(), "accessors").toFile()
@@ -402,22 +472,33 @@ abstract class GeneratePrecompiledScriptPluginAccessors @Inject internal constru
         }
 
     private
-    fun reportProjectSchemaError(plugins: List<PrecompiledScriptPlugin>, error: Throwable) {
-        if (strict.get()) throw GradleException(failedToGenerateAccessorsFor(plugins), error)
-        else logger.warn(failedToGenerateAccessorsFor(plugins), error)
+    fun reportProjectSchemaError(plugins: List<PrecompiledScriptPlugin>, stdout: String, stderr: String, error: Throwable) {
+        throw PrecompiledScriptException(failedToGenerateAccessorsFor(plugins, stdout, stderr), error)
     }
 
     private
-    fun failedToGenerateAccessorsFor(plugins: List<PrecompiledScriptPlugin>) =
-        plugins.joinToString(
-            prefix = "Failed to generate type-safe Gradle model accessors for the following precompiled script plugins:\n",
-            separator = "\n",
-            postfix = "\n"
-        ) { " - " + projectRelativePathOf(it) }
+    fun failedToGenerateAccessorsFor(plugins: List<PrecompiledScriptPlugin>, stdout: String, stderr: String): String =
+        buildString {
+            append(plugins.joinToString(
+                prefix = "Failed to generate type-safe Gradle model accessors for the following precompiled script plugins:\n",
+                separator = "\n",
+            ) { " - " + projectRelativePathOf(it) })
+            appendStdoutStderr(stdout, stderr)
+        }
+
+    private
+    fun StringBuilder.appendStdoutStderr(stdout: String, stderr: String) {
+        if (stdout.isNotBlank()) {
+            append("\nStandard output:\n${stdout.trim().prependIndent()}")
+        }
+        if (stderr.isNotBlank()) {
+            append("\nStandard error:\n${stderr.trim().prependIndent()}")
+        }
+    }
 
     private
     fun projectRelativePathOf(scriptPlugin: PrecompiledScriptPlugin) =
-        scriptPlugin.scriptFile.toRelativeString(projectLayout.projectDirectory.asFile)
+        normaliseFileSeparators(scriptPlugin.scriptFile.toRelativeString(projectLayout.projectDirectory.asFile))
 
     private
     fun IO.writeTypeSafeAccessorsFor(hashedSchema: HashedProjectSchema) {
@@ -473,3 +554,82 @@ fun ProjectInternal.applyPlugins(pluginRequests: PluginRequests) {
         classLoaderScope
     )
 }
+
+
+private
+fun <T> withCapturedOutputOnError(block: () -> T, onError: (ErrorWithCapturedOutput) -> T): T {
+    val outCapture = ThreadLocalCapturePrintStream(System.out)
+    val errCapture = ThreadLocalCapturePrintStream(System.err)
+    return try {
+        val previousOut = System.out
+        val previousErr = System.err
+        try {
+            System.setOut(outCapture)
+            System.setErr(errCapture)
+            block()
+        } finally {
+            System.out.flush()
+            System.setOut(previousOut)
+            outCapture.stop()
+            System.err.flush()
+            System.setErr(previousErr)
+            errCapture.stop()
+        }
+    } catch (error: Throwable) {
+        onError(ErrorWithCapturedOutput(error, outCapture.captureOutput.toString(), errCapture.captureOutput.toString()))
+    }
+}
+
+
+/**
+ * Captures output for the current thread, forward output to the original stream for other threads.
+ */
+private
+class ThreadLocalCapturePrintStream(originalOutput: PrintStream) : PrintStream(originalOutput) {
+
+    val captureOutput = ByteArrayOutputStream()
+
+    private
+    var isCapturing: ThreadLocal<Boolean>? = ThreadLocal.withInitial { false }
+
+    init {
+        isCapturing!!.set(true)
+    }
+
+    override fun write(buf: ByteArray, off: Int, len: Int) = safely {
+        if (doCapture) captureOutput.write(buf, off, len)
+        else super.write(buf, off, len)
+    }
+
+    override fun write(b: Int) = safely {
+        if (doCapture) captureOutput.write(b)
+        else super.write(b)
+    }
+
+    override fun flush() = safely {
+        captureOutput.flush()
+        super.flush()
+    }
+
+    fun stop() = safely {
+        isCapturing!!.remove()
+        // Give a chance for other threads' weak references to the local to be GCed if any
+        isCapturing = null
+    }
+
+    private
+    fun safely(block: () -> Unit) =
+        try {
+            synchronized(this, block)
+        } catch (ex: InterruptedException) {
+            Thread.currentThread().interrupt()
+        }
+
+    private
+    val doCapture: Boolean
+        get() = isCapturing != null && isCapturing!!.get()
+}
+
+
+private
+data class ErrorWithCapturedOutput(val error: Throwable, val stdout: String, val stderr: String)
