@@ -24,8 +24,8 @@ object KotlinDslPluginSupport {
             "-java-parameters",
             "-Xjvm-default=all",
             "-Xjsr305=strict",
+            "-Xjspecify-annotations=strict",
             "-Xsam-conversions=class",
-            "-XXLanguage:+DisableCompatibilityModeForNewInference",
-            "-XXLanguage:-TypeEnhancementImprovementsInStrictMode",
+            "-Xuse-fir-lt=false",
         )
 }
