@@ -17,8 +17,8 @@
 package org.gradle.internal.properties.annotations;
 
 import org.gradle.internal.reflect.annotations.AnnotationCategory;
+import org.jspecify.annotations.Nullable;
 
-import javax.annotation.Nullable;
 import java.lang.annotation.Annotation;
 import java.util.Map;
 
