@@ -18,8 +18,8 @@ package org.gradle.api.problems.internal;
 
 import org.gradle.api.problems.FileLocation;
 import org.gradle.api.problems.ProblemLocation;
+import org.jspecify.annotations.Nullable;
 
-import javax.annotation.Nullable;
 import java.util.List;
 
 /**
